@@ -1,22 +1,8 @@
-__all__ = ["AbstractScheduleModel", "HolidayModel"]
+__all__ = ["HolidayModel", "ScheduleAdjustment"]
 
 from java.lang import Object
 
 
-class AbstractScheduleModel(Object):
-    def getScheduleForDay(self, cal):
-        pass
-
-    def getType(self):
-        pass
-
-    def isObserveHolidays(self):
-        pass
-
-    def setObserveHolidays(self, observeHolidays):
-        pass
-
-
 class HolidayModel(Object):
     def __init__(self, name, date, repeatAnnually):
         self.name = name
@@ -31,3 +17,35 @@ class HolidayModel(Object):
 
     def isRepeatedAnnually(self):
         return self.repeatAnnually
+
+
+class ScheduleAdjustment(Object):
+    def __init__(self):
+        pass
+
+    def contains(self, timestamp):
+        pass
+
+    def getEnd(self):
+        pass
+
+    def getNote(self):
+        pass
+
+    def getStart(self):
+        pass
+
+    def isAvailable(self):
+        pass
+
+    def setAvailable(self, available):
+        pass
+
+    def setEnd(self, end):
+        pass
+
+    def setNoe(self, note):
+        pass
+
+    def setStart(self, start):
+        pass
