@@ -1,3 +1,5 @@
+from __future__ import print_function
+
 __all__ = ["SerialScriptModule"]
 
 from java.lang import Object
@@ -13,6 +15,16 @@ class SerialScriptModule(Object):
     def toString(self):
         pass
 
+    class PortManager(object):
+        def __init__(self):
+            pass
+
+        def __enter__(self):
+            print("Enter")
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            print("Exit")
+
     class SerialConfigurator(object):
         def setBitRate(self, value):
             pass
