@@ -9,11 +9,10 @@ This package provides a number of image producers, consumers, and
 filters that you can configure for your image processing needs.
 """
 
+from __future__ import print_function
 
 __all__ = ["BufferedImage"]
 
-import pprint
-
 from java.awt import Image
 
 
@@ -38,4 +37,4 @@ class BufferedImage(Image):
         Args:
             *args: Variable length argument list.
         """
-        pprint.pprint([self, args])
+        print(args)
