@@ -1,12 +1,12 @@
 # coding=utf-8
+
 """Package information."""
 
 __title__ = "ignition-api"
 __description__ = "Ignition Scripting API."
-__url__ = "https://github.com/thecesrom/Ignition/tree/7.9"
-__version__ = "7.9.18"
-__cycle__ = ".post1"
-__build__ = "2021062913"
+__url__ = "https://github.com/thecesrom/Ignition"
+__version__ = "8.1.10"
+__build__ = "2021090812"
 __author__ = "César Román"
 __author_email__ = "cesar@thecesrom.dev"
 __license__ = "MIT"
