@@ -4,13 +4,16 @@ The following functions give you access to view and interact with
 datasets.
 """
 
+from __future__ import print_function
+
 __all__ = [
+    "Dataset",
+    "PyDataSet",
     "addColumn",
     "addRow",
     "addRows",
     "appendDataset",
     "clearDataset",
-    "dataSetToExcel",
     "dataSetToHTML",
     "deleteRow",
     "deleteRows",
@@ -25,20 +28,142 @@ __all__ = [
     "sort",
     "toCSV",
     "toDataSet",
+    "toExcel",
     "toPyDataSet",
     "updateRow",
 ]
 
-import os.path
-import pprint
+import os
+from collections import Iterable
 
-from com.inductiveautomation.ignition.common import Dataset
-from com.inductiveautomation.ignition.common.script.builtin import (
-    DatasetUtilities,
-)
 from java.util import Locale
 
 
+class Dataset(Iterable):
+    """A dataset is a collection of values arranged in a structured
+    format.
+
+    Most datasets are two dimensional -- they can be viewed as a table
+    with rows and columns being the two dimensions. Values in a dataset
+    are usually accessed by specifying one index for each dimension of
+    data (row and column for tables).
+    """
+
+    def __iter__(self):
+        """Return the iterator object itself."""
+        pass
+
+    def getColumnCount(self):
+        """Returns the number of columns in the dataset.
+
+        Returns:
+            int: The number of columns in the dataset.
+        """
+        print(self)
+        return 4
+
+    def getColumnIndex(self, colName):
+        """Returns the index of the column with the name colName.
+
+        Args:
+            colName (str): The name of the column
+
+        Returns:
+            int: The index of the column with the name colName.
+        """
+        print(self, colName)
+        return 2
+
+    def getColumnName(self, colIndex):
+        """Returns the name of the column at the index colIndex.
+
+        Args:
+            colIndex (str): The index of the column.
+
+        Returns:
+            str: The name of the column at the index colIndex.
+        """
+        print(self, colIndex)
+        return "Population"
+
+    def getColumnNames(self):
+        """Returns a list with the names of all the columns.
+
+        Returns:
+            list[str]: A list with the names of all the columns.
+        """
+        print(self)
+        return ["City", "Population", "Timezone", "GMTOffset"]
+
+    def getColumnType(self, colIndex):
+        """Returns the type of the column at the index colIndex.
+
+        Args:
+            colIndex (int): The index of the column.
+
+        Returns:
+            object: The type of the column at the index colIndex.
+        """
+        print(self, colIndex)
+        return type(0)
+
+    def getColumnTypes(self):
+        """Returns a list with the types of all the columns.
+
+        Returns:
+             list[object]: A list with the types of all the columns.
+        """
+        print(self)
+        return [type(""), type(0), type(""), type(0)]
+
+    def getRowCount(self):
+        """Returns the number of rows in the dataset.
+
+        Returns:
+            int: The number of rows in the dataset.
+        """
+        print(self)
+        return 5
+
+    def getValueAt(self, *args):
+        """Returns the value at the specified row index and column name.
+
+        Args:
+            args (object): args[0] = rowIndex, args[1] colIndex or
+                colName.
+
+        Returns:
+            object: The value at the specified row index and column
+                name.
+        """
+        ret = None
+        if isinstance(args[1], int):
+            print(self, "colIndex")
+            ret = "PST"
+        elif isinstance(args[1], str):
+            print(self, "colName")
+            ret = 2853114
+        return ret
+
+
+class PyDataSet(Iterable):
+    """PyDatasets are special, in that they can be handled similarly to
+    other Python sequences.
+    """
+
+    def __add__(self, other):
+        """Add a new element."""
+        pass
+
+    def __iter__(self):
+        """Return the iterator object itself."""
+        pass
+
+    def __len__(self):
+        """Return the number of elements."""
+        pass
+
+
 def addColumn(dataset, colIndex, col, colName, colType):
     """Takes a dataset and returns a new dataset with a new column added
     or inserted into it.
@@ -70,7 +195,7 @@ def addColumn(dataset, colIndex, col, colName, colType):
     Returns:
         Dataset: A new dataset with the new column inserted or appended.
     """
-    pprint.pprint([dataset, colIndex, col, colName, colType])
+    print(dataset, colIndex, col, colName, colType)
     return Dataset()
 
 
@@ -100,7 +225,7 @@ def addRow(dataset, rowIndex, row):
     Returns:
         Dataset: A new dataset with the new row inserted or appended.
     """
-    pprint.pprint([dataset, rowIndex, row])
+    print(dataset, rowIndex, row)
     return Dataset()
 
 
@@ -130,7 +255,7 @@ def addRows(dataset, rowIndex, rows):
     Returns:
         Dataset: A new dataset with the new row inserted or appended.
     """
-    pprint.pprint([dataset, rowIndex, rows])
+    print(dataset, rowIndex, rows)
     return Dataset()
 
 
@@ -150,7 +275,7 @@ def appendDataset(dataset1, dataset2):
         Dataset: A new dataset that is a combination of the original two
             datasets.
     """
-    pprint.pprint([dataset1, dataset2])
+    print(dataset1, dataset2)
     return Dataset()
 
 
@@ -165,35 +290,10 @@ def clearDataset(dataset):
     Returns:
         Dataset: A new dataset with no data.
     """
-    pprint.pprint(dataset)
+    print(dataset)
     return Dataset()
 
 
-def dataSetToExcel(showHeaders, datasets, nullsEmpty=None):
-    """Formats the contents of one or more datasets as an excel
-    spreadsheet, returning the results as a string.
-
-    Each dataset specified will be added as a worksheet in the Excel
-    workbook. This function uses an xml-format for Excel spreadsheets,
-    not the native Excel file format.
-
-    Args:
-        showHeaders (bool): If True (1), the spreadsheet will include a
-            header row.
-        datasets (list[object]): A sequence of datasets, one for each
-            sheet in the resulting workbook.
-        nullsEmpty (bool): If True (1), the spreadsheet will leave
-            cells with NULL values empty, instead of allowing Excel to
-            provide a default value like 0. Defaults to False. Optional.
-
-    Returns:
-        str: An Excel-compatible XML-based workbook, with one worksheet
-            per dataset.
-    """
-    pprint.pprint([showHeaders, datasets, nullsEmpty])
-    return ""
-
-
 def dataSetToHTML(showHeaders, dataset, title):
     """Formats the contents of a dataset as an HTML page, returning the
     results as a string.
@@ -209,8 +309,8 @@ def dataSetToHTML(showHeaders, dataset, title):
     Returns:
         str: The HTML page as a string.
     """
-    pprint.pprint([showHeaders, dataset, title])
-    return "<html><head>%s</head><body>data</body></html>" % title
+    print(showHeaders, dataset, title)
+    return "<html><head>{}</head><body>data</body></html>".format(title)
 
 
 def deleteRow(dataset, rowIndex):
@@ -235,7 +335,7 @@ def deleteRow(dataset, rowIndex):
         IndexError: If rowIndex is less than zero or greater than the
             row count of the dataset -1.
     """
-    pprint.pprint([dataset, rowIndex])
+    print(dataset, rowIndex)
     if rowIndex < 0:
         raise IndexError("Error")
 
@@ -266,7 +366,7 @@ def deleteRows(dataset, rowIndices):
         IndexError: If any element is less than zero or greater than the
             number of rows in the dataset - 1.
     """
-    pprint.pprint([dataset, rowIndices])
+    print(dataset, rowIndices)
     if -1 in rowIndices:
         raise IndexError("Error")
 
@@ -287,15 +387,14 @@ def exportCSV(filename, showHeaders, dataset):
         str: The path to the saved file, or None if the action was
             canceled by the user.
     """
-    pprint.pprint([filename, showHeaders, dataset])
+    print(filename, showHeaders, dataset)
     return os.path.expanduser("~")
 
 
 def exportExcel(filename, showHeaders, dataset, nullsEmpty=False):
     """Exports the contents of a dataset as an Excel spreadsheet,
-    prompting the user to save the file to disk.
-
-    Uses the same format as the dataSetToExcel function.
+    prompting the user to save the file to disk. Uses the same format as
+    the toExcel function.
 
     Args:
         filename (str): A suggested filename to save as.
@@ -311,7 +410,7 @@ def exportExcel(filename, showHeaders, dataset, nullsEmpty=False):
         str: The path to the saved file, or None if the action was
             canceled by the user.
     """
-    pprint.pprint([filename, showHeaders, dataset, nullsEmpty])
+    print(filename, showHeaders, dataset, nullsEmpty)
     return os.path.expanduser("~")
 
 
@@ -331,7 +430,7 @@ def exportHTML(filename, showHeaders, dataset, title):
         str: The path to the saved file, or None if the action was
             canceled by the user.
     """
-    pprint.pprint([filename, showHeaders, dataset, title])
+    print(filename, showHeaders, dataset, title)
     return os.path.expanduser("~")
 
 
@@ -348,7 +447,7 @@ def filterColumns(dataset, columns):
     Returns:
         Dataset: A new dataset containing the filtered columns.
     """
-    pprint.pprint([dataset, columns])
+    print(dataset, columns)
     return Dataset()
 
 
@@ -367,7 +466,7 @@ def formatDates(dataset, dateFormat, locale=Locale.ENGLISH):
     Returns:
         Dataset: A new dataset, containing the formatted dates.
     """
-    pprint.pprint([dataset, dateFormat, locale])
+    print(dataset, dateFormat, locale)
     return Dataset()
 
 
@@ -387,7 +486,7 @@ def fromCSV(csv):
     Returns:
         Dataset: A new dataset.
     """
-    pprint.pprint(csv)
+    print(csv)
     return Dataset()
 
 
@@ -400,7 +499,7 @@ def getColumnHeaders(dataset):
     Returns:
         list[str]: A list of column header strings.
     """
-    pprint.pprint(dataset)
+    print(dataset)
     return []
 
 
@@ -427,7 +526,7 @@ def setValue(dataset, rowIndex, columnName, value):
          Dataset: A new dataset, with the new value set at the given
             location.
     """
-    pprint.pprint([dataset, rowIndex, columnName, value])
+    print(dataset, rowIndex, columnName, value)
     return Dataset()
 
 
@@ -447,7 +546,7 @@ def sort(dataset, keyColumn, ascending=True):
     Returns:
         Dataset: A new sorted dataset.
     """
-    pprint.pprint([dataset, keyColumn, ascending])
+    print(dataset, keyColumn, ascending)
     return Dataset()
 
 
@@ -473,7 +572,7 @@ def toCSV(dataset, showHeaders=True, forExport=False, localized=False):
     Returns:
         str: The CSV data as a string.
     """
-    pprint.pprint([dataset, showHeaders, forExport, localized])
+    print(dataset, showHeaders, forExport, localized)
     return ""
 
 
@@ -492,10 +591,41 @@ def toDataSet(*args):
     Returns:
         Dataset: The newly created dataset.
     """
-    pprint.pprint(args)
+    for arg in args:
+        print(arg)
     return Dataset()
 
 
+def toExcel(showHeaders, dataset, nullsEmpty=False, sheetNames=None):
+    """Formats the contents of one or more datasets as an excel
+    spreadsheet, returning the results as a string.
+
+    Each dataset specified will be added as a worksheet in the Excel
+    workbook. This function uses an xml-format for Excel spreadsheets,
+    not the native Excel file format.
+
+    Args:
+        showHeaders (bool): If True, the spreadsheet will include a
+            header row. If False, the header row will be omitted.
+        dataset (list[Dataset]): A sequence of one or more datasets,
+            one for each sheet in the resulting workbook.
+        nullsEmpty (bool): If True (1), the spreadsheet will leave cells
+            with NULL values empty, instead of allowing Excel to provide
+            a default value like 0. Defaults to False. Optional.
+        sheetNames (list[str]): Expects a list of strings, where each
+            string is a name for one of the datasets. When used, there
+            must be an equal number of string names in sheetName as
+            there are datasets in the dataset parameter. Names provided
+            in this parameter may be sanitized into acceptable Excel
+            sheet names. Optional.
+
+    Returns:
+        str: An Excel-compatible XML-based workbook, with one worksheet
+            per dataset.
+    """
+    print(showHeaders, dataset, nullsEmpty, sheetNames)
+
+
 def toPyDataSet(dataset):
     """This function converts from a normal DataSet to a PyDataSet,
     which is a wrapper class which makes working with datasets more
@@ -507,8 +637,8 @@ def toPyDataSet(dataset):
     Returns:
         PyDataSet: The newly created PyDataSet.
     """
-    pprint.pprint(dataset)
-    return DatasetUtilities.PyDataSet()
+    print(dataset)
+    return PyDataSet()
 
 
 def updateRow(dataset, rowIndex, changes):
@@ -536,5 +666,5 @@ def updateRow(dataset, rowIndex, changes):
          Dataset: A new dataset with the values at the specified row
             updated according to the values in the dictionary.
     """
-    pprint.pprint([dataset, rowIndex, changes])
+    print(dataset, rowIndex, changes)
     return Dataset()
