@@ -3,7 +3,6 @@
 The following functions give you access to test and modify dates.
 """
 
-
 __all__ = [
     "addDays",
     "addHours",
@@ -50,7 +49,6 @@ __all__ = [
     "yearsBetween",
 ]
 
-import pprint
 from calendar import monthrange
 from datetime import datetime, timedelta
 from time import localtime, mktime
@@ -206,7 +204,7 @@ def daysBetween(date_1, date_2):
     return (date_2 - date_1).days
 
 
-def format(date, format):
+def format(date, format="yyyy-MM-dd HH:mm:ss"):
     """Returns the given date as a string, formatted according to a
     pattern.
 
@@ -217,6 +215,7 @@ def format(date, format):
     Args:
         date (Date): The date to format.
         format (str): A format string such as "yyyy-MM-dd HH:mm:ss".
+            Optional.
 
     Returns:
         str: A string representing the formatted datetime
@@ -296,9 +295,8 @@ def getDate(year, month, day):
 
 
 def getDayOfMonth(date):
-    """Extracts the day of the month from a date.
-
-    The first day of the month is day 1.
+    """Extracts the day of the month from a date. The first day of the
+    month is day 1.
 
     Args:
         date (Date): The date to use.
@@ -325,9 +323,8 @@ def getDayOfWeek(date):
 
 
 def getDayOfYear(date):
-    """Extracts the day of the year from a date.
-
-    The first day of the year is day 1.
+    """Extracts the day of the year from a date. The first day of the
+    year is day 1.
 
     Args:
         date (Date): The date to use.
@@ -339,9 +336,8 @@ def getDayOfYear(date):
 
 
 def getHour12(date):
-    """Extracts the hour from a date.
-
-    Uses a 12 hour clock, so noon and midnight are returned as 0.
+    """Extracts the hour from a date. Uses a 12 hour clock, so noon and
+    midnight are returned as 0.
 
     Args:
         date (Date): The date to use.
@@ -353,9 +349,8 @@ def getHour12(date):
 
 
 def getHour24(date):
-    """Extracts the hour from a date.
-
-    Uses a 24 hour clock, so midnight is zero.
+    """Extracts the hour from a date. Uses a 24 hour clock, so midnight
+    is zero.
 
     Args:
         date (Date): The date to use.
@@ -585,8 +580,8 @@ def millisBetween(date_1, date_2):
             between two dates.
     """
     diff = date_2 - date_1
-    d, s, micro = diff.days, diff.seconds, diff.microseconds
-    return d * 86400 * 1000 + s * 1000 + micro // 1000
+    days, seconds, microseconds = diff.days, diff.seconds, diff.microseconds
+    return days * 86400 * 1000 + seconds * 1000 + microseconds // 1000
 
 
 def minutesBetween(date_1, date_2):
@@ -606,9 +601,8 @@ def minutesBetween(date_1, date_2):
 
 
 def monthsBetween(date_1, date_2):
-    """Calculates the number of whole months between two dates.
-
-    Daylight Saving Time changes are taken into account.
+    """Calculates the number of whole months between two dates. Daylight
+    Saving Time changes are taken into account.
 
     Args:
         date_1 (Date): The first date to use.
@@ -642,9 +636,8 @@ def now():
 def parse(
     dateString, formatString="yyyy-MM-dd HH:mm:ss", locale=Locale.ENGLISH
 ):
-    """Attempts to parse a string and create a Date.
-
-    Causes ParseException if the date dateString parameter is in an
+    """Attempts to parse a string and create a Date. Causes
+    ParseException if the date dateString parameter is in an
     unrecognized format.
 
     Args:
@@ -658,7 +651,7 @@ def parse(
     Returns:
         Date: The parsed date.
     """
-    pprint.pprint([dateString, formatString, locale])
+    print(dateString, formatString, locale)
     return now()
 
 
@@ -725,9 +718,8 @@ def weeksBetween(date_1, date_2):
 
 
 def yearsBetween(date_1, date_2):
-    """Calculates the number of whole years between two dates.
-
-    Daylight Saving Time changes are taken into account.
+    """Calculates the number of whole years between two dates. Daylight
+    Saving Time changes are taken into account.
 
     Args:
         date_1 (Date): The first date to use.
