@@ -4,8 +4,11 @@ The following functions give you access to view and modify data in the
 database.
 """
 
+from __future__ import print_function
+
 __all__ = [
     "addDatasource",
+    "beginNamedQueryTransaction",
     "beginTransaction",
     "clearAllNamedQueryCaches",
     "clearNamedQueryCache",
@@ -14,9 +17,6 @@ __all__ = [
     "createSProcCall",
     "dateFormat",
     "execSProcCall",
-    "exportCSV",
-    "exportExcel",
-    "exportHTML",
     "getConnectionInfo",
     "getConnections",
     "refresh",
@@ -26,6 +26,7 @@ __all__ = [
     "runPrepQuery",
     "runPrepUpdate",
     "runQuery",
+    "runSFNamedQuery",
     "runSFPrepUpdate",
     "runSFUpdateQuery",
     "runScalarPrepQuery",
@@ -36,14 +37,9 @@ __all__ = [
     "setDatasourceMaxConnections",
 ]
 
-import pprint
-
-from com.inductiveautomation.ignition.common import Dataset
-from com.inductiveautomation.ignition.common.script.builtin import (
-    DatasetUtilities,
-    SProcCall,
-)
+from java.lang import Object
 from javax.swing import JComponent
+from system.dataset import Dataset, PyDataSet
 
 # Type codes
 # These are codes defined by the JDBC specification.
@@ -94,15 +90,97 @@ REPEATABLE_READ = 4
 SERIALIZABLE = 8
 
 
+class SProcCall(Object):
+    def __init__(self):
+        pass
+
+    def getResultSet(self):
+        """Returns a dataset that is the resulting data of the stored
+        procedure, if any.
+
+        Returns:
+            Dataset: The dataset that is the resulting data of the
+                stored procedure, if any.
+        """
+        print(self)
+        return Dataset()
+
+    def getUpdateCount(self):
+        """Returns the number of rows modified by the stored procedure,
+        or -1 if not applicable.
+
+        Returns:
+             int: The number of rows modified by the stored procedure,
+                or -1 if not applicable.
+        """
+        print(self)
+        return 1
+
+    def getReturnValue(self):
+        """Returns the return value, if registerReturnParam had been
+        called.
+
+        Returns:
+             int: The return value, if registerReturnParam had been
+                called.
+        """
+        print(self)
+        return 0
+
+    def getOutParamValue(self, param):
+        """Returns the value of the previously registered out-parameter.
+
+        Args:
+            param (object): Index (int) or name (str) of the previously
+                registered out-parameter.
+
+        Returns:
+            object: The value of the previously registered
+                out-parameter.
+        """
+        print(self, param)
+        return 0
+
+    def registerInParam(self, param, typeCode, value):
+        """Registers an in parameter for the stored procedure.
+
+        Args:
+            param (object): Index (int starting at 1, not 0), or name
+                (str).
+            typeCode (int): Type code constant.
+            value (object): Value of type typeCode.
+        """
+        print(self, param, typeCode, value)
+
+    def registerOutParam(self, param, typeCode):
+        """Registers an out parameter for the stored procedure.
+
+        Args:
+            param (object): Index (int starting at 1, not 0), or name
+                (str).
+            typeCode (int): Type code constant.
+        """
+        print(self, param, typeCode)
+
+    def registerReturnParam(self, typeCode):
+        """Use this function to specify the datatype of the returned
+        value.
+
+        Args:
+            typeCode (int): Type code constant.
+        """
+        print(self, typeCode)
+
+
 def addDatasource(
     jdbcDriver,
     name,
-    description,
-    connectUrl,
-    username,
-    password,
-    props,
-    validationQuery,
+    description=None,
+    connectUrl=None,
+    username=None,
+    password=None,
+    props=None,
+    validationQuery=None,
     maxConnections=8,
 ):
     """Adds a new database connection in Ignition.
@@ -111,41 +189,81 @@ def addDatasource(
         jdbcDriver (str): The name of the JDBC driver in Ignition.
             Required.
         name (str): The datasource name. Required.
-        description (str): Description of the datasource.
+        description (str): Description of the datasource. Optional.
         connectUrl (str): Default is the connect URL for JDBC driver.
         username (str): Username to login to the datasource with.
-        password (str): Password for the login.
-        props (str): The extra connection parameters.
+            Optional.
+        password (str): Password for the login. Optional.
+        props (str): The extra connection parameters. Optional.
         validationQuery (str): Default is the validation query for the
-            JDBC driver.
-        maxConnections (int): Default is 8.
+            JDBC driver. Optional.
+        maxConnections (int): Default is 8. Optional.
     """
-    pprint.pprint(
-        [
-            jdbcDriver,
-            name,
-            description,
-            connectUrl,
-            username,
-            password,
-            props,
-            validationQuery,
-            maxConnections,
-        ]
+    print(
+        jdbcDriver,
+        name,
+        description,
+        connectUrl,
+        username,
+        password,
+        props,
+        validationQuery,
+        maxConnections,
     )
 
 
-def beginTransaction(database=None, isolationLevel=None, timeout=None):
-    """Begins a new database transaction. Database transactions are used
-    to execute multiple queries in an atomic fashion.
+def beginNamedQueryTransaction(*args):
+    """Begins a new database transaction using Named Queries.
 
-    After executing queries, you must either commit the transaction to
-    have your changes take effect, or rollback the transaction which
-    will make all operations since the last commit not take place. The
-    transaction is given a new unique string code, which is then
-    returned. You can then use this code as the tx argument for other
-    system.db.* function calls to execute various types of queries using
-    this transaction.
+    Database transactions are used to execute multiple queries in an
+    atomic fashion. After executing queries, you must either commit the
+    transaction to have your changes take effect, or rollback the
+    transaction which will make all operations since the last commit not
+    take place. The transaction is given a new unique string code, which
+    is then returned. You can then use this code as the tx argument for
+    other system.db.* function calls to execute various types of queries
+    using this transaction.
+
+    An open transaction consumes one database connection until it is
+    closed. Because leaving connections open indefinitely would exhaust
+    the connection pool, each transaction is given a timeout. Each time
+    the transaction is used, the timeout timer is reset. For example, if
+    you make a transaction with a timeout of one minute, you must
+    complete that transaction within a minute. If a transaction is
+    detected to have timed out, it will be automatically closed and its
+    transaction id will no longer be valid.
+
+    When calling from Vision use:
+    system.db.beginNamedQueryTransaction([database], [isolationLevel],
+                                         [timeout])
+
+    When calling from Perspective or Gateway scope use:
+    system.db.beginNamedQueryTransaction(project, database,
+                                        [isolationLevel], [timeout])
+
+    Args:
+        *args: Variable length argument list.
+
+    Returns:
+         str: The new transaction ID. You'll use this ID as the "tx"
+            argument for all other calls to have them execute against
+            this transaction.
+    """
+    print(args)
+    return "transaction_id"
+
+
+def beginTransaction(database=None, isolationLevel=None, timeout=None):
+    """Begins a new database transaction.
+
+    Database transactions are used to execute multiple queries in an
+    atomic fashion. After executing queries, you must either commit the
+    transaction to have your changes take effect, or rollback the
+    transaction which will make all operations since the last commit not
+    take place. The transaction is given a new unique string code, which
+    is then returned. You can then use this code as the tx argument for
+    other system.db.* function calls to execute various types of queries
+    using this transaction.
 
     An open transaction consumes one database connection until it is
     closed. Because leaving connections open indefinitely would exhaust
@@ -175,7 +293,7 @@ def beginTransaction(database=None, isolationLevel=None, timeout=None):
             argument for all other calls to have them execute against
             this transaction.
     """
-    pprint.pprint([database, isolationLevel, timeout])
+    print(database, isolationLevel, timeout)
     return "transaction_id"
 
 
@@ -195,7 +313,7 @@ def clearAllNamedQueryCaches(*args):
     Args:
         *args: Variable length argument list.
     """
-    pprint.pprint(args)
+    print(args)
 
 
 def clearNamedQueryCache(*args):
@@ -214,7 +332,7 @@ def clearNamedQueryCache(*args):
     Args:
         *args: Variable length argument list.
     """
-    pprint.pprint(args)
+    print(args)
 
 
 def closeTransaction(tx):
@@ -227,7 +345,7 @@ def closeTransaction(tx):
     Args:
         tx (str): The transaction ID.
     """
-    pprint.pprint(tx)
+    print(tx)
 
 
 def commitTransaction(tx):
@@ -243,7 +361,7 @@ def commitTransaction(tx):
     Args:
         tx (str): The transaction ID.
     """
-    pprint.pprint(tx)
+    print(tx)
 
 
 def createSProcCall(procedureName, database=None, tx=None, skipAudit=None):
@@ -255,8 +373,8 @@ def createSProcCall(procedureName, database=None, tx=None, skipAudit=None):
         database (str): The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        tx (str): A transaction identifier. If omitted, the call will
-            be executed in its own transaction. Optional.
+        tx (str): A transaction identifier. If omitted, the call will be
+            executed in its own transaction. Optional.
         skipAudit (bool): A flag which, if set to True, will cause the
             procedure call to skip the audit system. Useful for some
             queries that have fields which won't fit into the audit log.
@@ -267,7 +385,7 @@ def createSProcCall(procedureName, database=None, tx=None, skipAudit=None):
             configured and then used as the argument to
             system.db.execSProcCall.
     """
-    pprint.pprint([procedureName, database, tx, skipAudit])
+    print(procedureName, database, tx, skipAudit)
     return SProcCall()
 
 
@@ -284,7 +402,7 @@ def dateFormat(date, formatPattern):
         str: The date as a string formatted according to the format
             pattern.
     """
-    pprint.pprint([date, formatPattern])
+    print(date, formatPattern)
     return ""
 
 
@@ -300,75 +418,16 @@ def execSProcCall(callContext):
             configured. Use system.db.createSProcCall to create a call
             context.
     """
-    pprint.pprint(callContext)
-
-
-def exportCSV(filename, showHeaders, dataset):
-    """Exports the contents of a dataset as a CSV file, prompting the
-    user to save the file to disk.
-
-    Args:
-        filename (str): A suggested filename to save as.
-        showHeaders (bool): If True (1), the CSV file will include a
-            header row.
-        dataset (Dataset): The dataset to export.
-
-    Returns:
-        str: The path to the saved file, or None if the action was
-            canceled by the user.
-    """
-    pprint.pprint([filename, showHeaders, dataset])
-
-
-def exportExcel(filename, showHeaders, dataset, nullsEmpty=False):
-    """Exports the contents of a dataset as an Excel spreadsheet,
-    prompting the user to save the file to disk.
-
-    Uses the same format as the dataSetToExcel function.
-
-    Args:
-        filename (str): A suggested filename to save as.
-        showHeaders (bool): If True (1), the spreadsheet will include a
-            header row.
-        dataset list[object]: A sequence of datasets, one for each sheet
-            in the resulting workbook.
-        nullsEmpty (bool): If True (1), the spreadsheet will leave cells
-            with NULL values empty. Optional.
-
-    Returns:
-        str: The path to the saved file, or None if the action was
-            canceled by the user.
-    """
-    pprint.pprint([filename, showHeaders, dataset, nullsEmpty])
-
-
-def exportHTML(filename, showHeaders, dataset, title):
-    """Exports the contents of a dataset to an HTML page.
-
-    Prompts the user to save the file to disk.
-
-    Args:
-        filename (str): A suggested filename to save as.
-        showHeaders (bool): If True (1), the HTML table will include a
-            header row.
-        dataset (Dataset): The dataset to export.
-        title (str): The title for the HTML page.
-
-    Returns:
-        str: The path to the saved file, or None if the action was
-            canceled by the user.
-    """
-    pprint.pprint([filename, showHeaders, dataset, title])
+    print(callContext)
 
 
 def getConnectionInfo(name=""):
-    """Returns a dataset of information about a single database
-    connection, as specified by the name argument, or about the current
-    project's default database connection.
+    """Returns a dataset of information about a database connection.
 
     Args:
         name (str): The name of the database connection to find
-            information about. Optional.
+            information about, or the project's default connection if
+            omitted. Optional.
 
     Returns:
         Dataset: A dataset containing information about the named
@@ -376,7 +435,7 @@ def getConnectionInfo(name=""):
             database connection, or an empty dataset if the connection
             wasn't found.
     """
-    pprint.pprint(name)
+    print(name)
     return Dataset()
 
 
@@ -413,7 +472,7 @@ def refresh(component, propertyName):
         bool: True (1) if the property was found and refreshed
             successfully.
     """
-    pprint.pprint([component, propertyName])
+    print(component, propertyName)
     return True
 
 
@@ -423,7 +482,7 @@ def removeDatasource(name):
     Args:
         name (str): The name of the database connection in Ignition.
     """
-    pprint.pprint(name)
+    print(name)
 
 
 def rollbackTransaction(tx):
@@ -437,7 +496,7 @@ def rollbackTransaction(tx):
     Args:
         tx (str): The transaction ID.
     """
-    pprint.pprint(tx)
+    print(tx)
 
 
 def runNamedQuery(*args):
@@ -447,10 +506,10 @@ def runNamedQuery(*args):
     scope. Both versions of the function are listed below.
 
     When calling from the Project Scope use:
-    system.db.runNamedQuery(path, parameters)
+    system.db.runNamedQuery(path, parameters, [tx], [getKey])
 
     When calling from the Gateway Scope use:
-    system.db.runNamedQuery(project, path, parameters)
+    system.db.runNamedQuery(project, path, parameters, [tx], [getKey])
 
     Args:
         *args: Variable length argument list.
@@ -463,7 +522,7 @@ def runNamedQuery(*args):
             Query, or an object matching the datatype of the value
             returned by a Scalar Query.
     """
-    pprint.pprint(args)
+    print(args)
 
 
 def runPrepQuery(query, args, database="", tx=None):
@@ -499,8 +558,8 @@ def runPrepQuery(query, args, database="", tx=None):
     Returns:
         PyDataSet: The results of the query as a PyDataSet.
     """
-    pprint.pprint([query, args, database, tx])
-    return DatasetUtilities.PyDataSet()
+    print(query, args, database, tx)
+    return PyDataSet()
 
 
 def runPrepUpdate(
@@ -543,7 +602,7 @@ def runPrepUpdate(
             that was generated, depending on the value of the getKey
             flag.
     """
-    pprint.pprint([query, args, database, tx, getKey, skipAudit])
+    print(query, args, database, tx, getKey, skipAudit)
     return 1
 
 
@@ -567,8 +626,32 @@ def runQuery(query, database="", tx=None):
     Returns:
         PyDataSet: The results of the query as a PyDataSet.
     """
-    pprint.pprint([query, database, tx])
-    return DatasetUtilities.PyDataSet()
+    print(query, database, tx)
+    return PyDataSet()
+
+
+def runSFNamedQuery(*args):
+    """Runs a named query that goes through the Store and Forward
+    system.
+
+    Note that the number of parameters in the function is determined by
+    scope.
+
+    When calling from the Project Scope use:
+    system.db.runSFNamedQuery(path, parameters, [getKey])
+
+    When calling from the Gateway Scope use:
+    system.db.runSFNamedQuery(project, path, parameters, [getKey])
+
+    Args:
+        *args : Variable length argument list.
+
+    Returns:
+        bool: Returns True if successfully sent to the Store and Forward
+            system.
+    """
+    print(args)
+    return True
 
 
 def runSFPrepUpdate(query, args, datasources):
@@ -596,7 +679,7 @@ def runSFPrepUpdate(query, args, datasources):
         bool: Returns True if successfully sent to store-and-forward
             system.
     """
-    pprint.pprint([query, args, datasources])
+    print(query, args, datasources)
     return True
 
 
@@ -613,7 +696,7 @@ def runSFUpdateQuery(query, datasources):
     Returns:
         bool: Returns True if successful and False if not.
     """
-    pprint.pprint([query, datasources])
+    print(query, datasources)
     return True
 
 
@@ -642,10 +725,10 @@ def runScalarPrepQuery(query, args, database="", tx=None):
          object: The value from the first row and first column of the
             results. Returns None if no rows were returned.
     """
-    pprint.pprint([query, args, database, tx])
+    print(query, args, database, tx)
 
 
-def runScalarQuery(query, database, tx):
+def runScalarQuery(query, database="", tx=None):
     """Runs a query against a database connection just like the runQuery
     function, but only returns the value from the first row and column.
 
@@ -665,7 +748,7 @@ def runScalarQuery(query, database, tx):
          object: The value from the first row and first column of the
             results. Returns None if no rows were returned.
     """
-    pprint.pprint([query, database, tx])
+    print(query, database, tx)
 
 
 def runUpdateQuery(query, database="", tx=None, getKey=False, skipAudit=True):
@@ -702,7 +785,7 @@ def runUpdateQuery(query, database="", tx=None, getKey=False, skipAudit=True):
             that was generated, depending on the value of the getKey
             flag.
     """
-    pprint.pprint([query, database, tx, getKey, skipAudit])
+    print(query, database, tx, getKey, skipAudit)
     return 1
 
 
@@ -713,7 +796,7 @@ def setDatasourceConnectURL(name, connectUrl):
         name (str): The name of the database connection in Ignition.
         connectUrl (str): The new connect URL.
     """
-    pprint.pprint([name, connectUrl])
+    print(name, connectUrl)
 
 
 def setDatasourceEnabled(name, enabled):
@@ -724,7 +807,7 @@ def setDatasourceEnabled(name, enabled):
         enabled (bool): True if the connection should be enabled, False
             otherwise.
     """
-    pprint.pprint([name, enabled])
+    print(name, enabled)
 
 
 def setDatasourceMaxConnections(name, maxConnections):
@@ -735,4 +818,4 @@ def setDatasourceMaxConnections(name, maxConnections):
         name (str): The name of the database connection in Ignition.
         maxConnections (int): The number of maximum connections allowed.
     """
-    pprint.pprint([name, maxConnections])
+    print(name, maxConnections)
