@@ -4,17 +4,15 @@ The following functions give you access to import and remove Transaction
 Groups.
 """
 
+from __future__ import print_function
 
 __all__ = ["loadFromFile", "removeGroups"]
 
-import pprint
-
 
 def loadFromFile(filePath, projectName, mode):
     """Loads a transaction group configuration from an xml export, into
-    the specified project (creating the project if necessary).
-
-    The mode parameter dictates how overwrites occur.
+    the specified project (creating the project if necessary). The mode
+    parameter dictates how overwrites occur.
 
     Args:
         filePath (str): The path to a valid transaction group xml or csv
@@ -23,14 +21,12 @@ def loadFromFile(filePath, projectName, mode):
         mode (int): How duplicates will be handled. 0 = Overwrite,
             1 = Ignore, 2 = Replace the existing project with this one.
     """
-    pprint.pprint([filePath, projectName, mode])
+    print(filePath, projectName, mode)
 
 
 def removeGroups(projectName, paths):
-    """Removes the specified groups from the project.
-
-    The group paths are "Folder/Path/To/GroupName", separated by forward
-    slashes.
+    """Removes the specified groups from the project. The group paths
+    are "Folder/Path/To/GroupName", separated by forward slashes.
 
     Args:
         projectName (str): The project to remove from. If the project
@@ -39,4 +35,4 @@ def removeGroups(projectName, paths):
             paths are "Folder/Path/To/GroupName", separated by forward
             slashes.
     """
-    pprint.pprint([projectName, paths])
+    print(projectName, paths)
