@@ -4,6 +4,8 @@ The following functions allow you to control windows and create popup
 interfaces.
 """
 
+from __future__ import print_function
+
 __all__ = [
     "chooseColor",
     "closeDesktop",
@@ -20,6 +22,7 @@ __all__ = [
     "getOpenedWindows",
     "getParentWindow",
     "getQuality",
+    "getScreenIndex",
     "getScreens",
     "getSibling",
     "getWindow",
@@ -38,14 +41,19 @@ __all__ = [
     "warningBox",
 ]
 
-import pprint
-
-from com.inductiveautomation.factorypmi.application import FPMIWindow
-from com.inductiveautomation.factorypmi.application.script.builtin import (
-    WindowUtilities,
-)
 from java.awt import Color
-from javax.swing import JComponent, JFrame, JPopupMenu
+from java.lang import Object
+from java.util import EventObject
+from javax.swing import (
+    JComponent,
+    JFrame,
+    JInternalFrame,
+    JLabel,
+    JOptionPane,
+    JPanel,
+    JPopupMenu,
+    JTextField,
+)
 
 # Constants
 ACCL_NONE = 0
@@ -57,8 +65,59 @@ COORD_SCREEN = 0
 COORD_DESIGNER = 1
 
 
+class FPMIWindow(JInternalFrame):
+    """FPMIWindow object."""
+
+    # Fields.
+    CACHE_ALWAYS = 2
+    CACHE_AUTO = 0
+    CACHE_NEVER = 1
+    DOCK_EAST = 2
+    DOCK_FLOAT = 0
+    DOCK_NORTH = 2
+    DOCK_SOUTH = 4
+    DOCK_WEST = 3
+    PARENT_WINDOW_NAME = "_parent"
+    SHOW_ALWAYS = 0
+    SHOW_NEVER = 1
+    SHOW_MAXIMIZED = 2
+
+    _path = "Path/To/Window"
+
+    def __init__(self, name):
+        self.name = name
+
+    def getPath(self):
+        return self._path
+
+    def getRootContainer(self):
+        print(self)
+
+
+class WindowUtilities(Object):
+    """WindowUtilities object."""
+
+    def confirm(self, *args):
+        pass
+
+    def errorBox(self, *args):
+        pass
+
+    def inputBox(self, *args):
+        pass
+
+    def messageBox(self, *args):
+        pass
+
+    def passwordBox(self, *args):
+        pass
+
+    def warningBox(self, *args):
+        pass
+
+
 def _dummy(message, title):
-    pprint.pprint([message, title])
+    print(message, title)
 
 
 def chooseColor(initialColor, dialogTitle="Choose Color"):
@@ -74,7 +133,7 @@ def chooseColor(initialColor, dialogTitle="Choose Color"):
     Returns:
         Color: The new color chosen by the user.
     """
-    pprint.pprint([initialColor, dialogTitle])
+    print(initialColor, dialogTitle)
     return Color()
 
 
@@ -89,12 +148,13 @@ def closeDesktop(handle="primary"):
             Alternatively, the handle "primary" can be used to refer to
             the Primary Desktop.
     """
-    pprint.pprint(handle)
+    print(handle)
 
 
 def color(*args):
     """Creates a new color object, either by parsing a string or by
-    having the RGB[A] channels specified explicitly.
+    having the RGB[A] channels specified explicitly. See toColor to see
+    a list of available color names.
 
     Args:
         args: Variable-length argument list.
@@ -102,7 +162,7 @@ def color(*args):
     Returns:
         Color: The newly created color.
     """
-    pprint.pprint(args)
+    print(args)
 
 
 def confirm(message, title="Confirm", allowCancel=False):
@@ -119,14 +179,33 @@ def confirm(message, title="Confirm", allowCancel=False):
         bool: True (1) if the user selected "Yes", False (0) if the user
             selected "No", None if the user selected "Cancel".
     """
-    pprint.pprint([message, title, allowCancel])
-    return True
+    options = ["Yes", "No"]
+
+    if allowCancel:
+        options.append("Cancel")
+
+    choice = JOptionPane.showOptionDialog(
+        None,
+        message,
+        title,
+        JOptionPane.YES_NO_CANCEL_OPTION,
+        JOptionPane.QUESTION_MESSAGE,
+        None,
+        options,
+        options[0],
+    )
+
+    return (
+        not bool(choice)
+        if choice in [JOptionPane.YES_OPTION, JOptionPane.NO_OPTION]
+        else None
+    )
 
 
 def convertPointToScreen(x, y, event):
-    """Converts a pair of coordinates that are relative to
-    the upper-left corner of some component to be relative to the
-    upper-left corner of the entire screen.
+    """Converts a pair of coordinates that are relative to the
+    upper-left corner of some component to be relative to the upper-left
+    corner of the entire screen.
 
     Args:
         x (int): The X-coordinate, relative to the component that fired
@@ -138,7 +217,7 @@ def convertPointToScreen(x, y, event):
     Returns:
         tuple: A tuple of (x,y) in screen coordinates.
     """
-    pprint.pprint([x, y, event])
+    print(x, y, event)
     return x, y
 
 
@@ -155,7 +234,7 @@ def createPopupMenu(itemNames, itemFunctions):
     Returns:
         JPopupMenu: The javax.swing.JPopupMenu that was created.
     """
-    pprint.pprint([itemNames, itemFunctions])
+    print(itemNames, itemFunctions)
     return JPopupMenu()
 
 
@@ -173,7 +252,7 @@ def desktop(handle="primary"):
         WindowUtilities: A copy of system.gui that will be relative to
             the desktop named by the given handle.
     """
-    pprint.pprint(handle)
+    print(handle)
     return WindowUtilities()
 
 
@@ -184,7 +263,9 @@ def errorBox(message, title="Error"):
         message (str): The message to display in an error box.
         title (str): The title for the error box. Optional.
     """
-    _dummy(message, title)
+    JOptionPane.showMessageDialog(
+        None, message, title, JOptionPane.ERROR_MESSAGE
+    )
 
 
 def findWindow(path):
@@ -202,7 +283,7 @@ def findWindow(path):
             is not open, or have more than one entry if multiple windows
             are open.
     """
-    pprint.pprint(path)
+    print(path)
     return []
 
 
@@ -270,7 +351,7 @@ def getParentWindow(event):
         object: The window that contains the component that fired the
             event.
     """
-    pprint.pprint(event)
+    print(event)
     return object
 
 
@@ -290,10 +371,20 @@ def getQuality(component, propertyName):
     Returns:
         int: The data quality of the given property as an integer.
     """
-    pprint.pprint([component, propertyName])
+    print(component, propertyName)
     return 192
 
 
+def getScreenIndex():
+    """Returns the returns an integer value representing the current
+    screen index based on the screen this function was called from.
+
+    Returns:
+        int: The screen that the function was called from.
+    """
+    return 0
+
+
 def getScreens():
     """Get a list of all the monitors on the computer this client is
     open on.
@@ -320,7 +411,7 @@ def getSibling(event, name):
     Returns:
         object: The sibling component itself.
     """
-    pprint.pprint([event, name])
+    print(event, name)
     return FPMIWindow("Sibling")
 
 
@@ -337,7 +428,7 @@ def getWindow(name):
             .getRootContainer() to grab the root container of the
             window.
     """
-    pprint.pprint(name)
+    print(name)
     return FPMIWindow("Main Window")
 
 
@@ -370,7 +461,27 @@ def inputBox(message, defaultText=None):
     Returns:
         str: The string value that was entered in the input box.
     """
-    pprint.pprint([message, defaultText])
+    options = ["OK", "Cancel"]
+
+    panel = JPanel()
+    label = JLabel("{}: ".format(message))
+    panel.add(label)
+    text_field = JTextField(25)
+    text_field.setText(defaultText)
+    panel.add(text_field)
+
+    choice = JOptionPane.showOptionDialog(
+        None,
+        panel,
+        "Input",
+        JOptionPane.OK_CANCEL_OPTION,
+        JOptionPane.QUESTION_MESSAGE,
+        None,
+        options,
+        options[0],
+    )
+
+    return text_field.getText() if choice == JOptionPane.OK_OPTION else None
 
 
 def isTouchscreenModeEnabled():
@@ -392,7 +503,9 @@ def messageBox(message, title="Information"):
             formatting.
         title (str): The title for the message box. Optional.
     """
-    _dummy(message, title)
+    JOptionPane.showMessageDialog(
+        None, message, title, JOptionPane.INFORMATION_MESSAGE
+    )
 
 
 def openDesktop(
@@ -431,7 +544,7 @@ def openDesktop(
     Returns:
         JFrame: A reference to the new Desktop frame.
     """
-    pprint.pprint([screen, handle, title, width, height, x, y, windows])
+    print(screen, handle, title, width, height, x, y, windows)
     return JFrame()
 
 
@@ -465,7 +578,7 @@ def passwordBox(message, title="Password", echoChar="*"):
         str: The password that was entered, or None if the prompt was
             canceled.
     """
-    pprint.pprint([message, title, echoChar])
+    print(message, title, echoChar)
     return "password"
 
 
@@ -478,7 +591,7 @@ def setScreenIndex(index):
         index (int): The new monitor index for this client to move to. 0
             based.
     """
-    pprint.pprint(index)
+    print(index)
 
 
 def setTouchscreenModeEnabled(enabled):
@@ -488,7 +601,7 @@ def setTouchscreenModeEnabled(enabled):
         enabled (bool): The new value for touchscreen mode being
             enabled.
     """
-    pprint.pprint(enabled)
+    print(enabled)
 
 
 def showNumericKeypad(initialValue=None, fontSize=None, usePasswordMode=False):
@@ -508,7 +621,7 @@ def showNumericKeypad(initialValue=None, fontSize=None, usePasswordMode=False):
     Returns:
         object: The value that was entered in the keypad.
     """
-    pprint.pprint([initialValue, fontSize, usePasswordMode])
+    print(initialValue, fontSize, usePasswordMode)
     return 43
 
 
@@ -530,7 +643,7 @@ def showTouchscreenKeyboard(
     Returns:
         str: The text that was "typed" in the on-screen keyboard.
     """
-    pprint.pprint([initialText, fontSize, passwordMode])
+    print(initialText, fontSize, passwordMode)
     return ""
 
 
@@ -586,19 +699,17 @@ def transform(
         object: An animation object that the script can use to pause(),
             resume(), or cancel() the transformation.
     """
-    pprint.pprint(
-        [
-            component,
-            newX,
-            newY,
-            newWidth,
-            newHeight,
-            duration,
-            callback,
-            framesPerSecond,
-            acceleration,
-            coordSpace,
-        ]
+    print(
+        component,
+        newX,
+        newY,
+        newWidth,
+        newHeight,
+        duration,
+        callback,
+        framesPerSecond,
+        acceleration,
+        coordSpace,
     )
 
 
@@ -610,4 +721,6 @@ def warningBox(message, title="Warning"):
             accept html formatting.
         title (str): The title for the warning box. Optional.
     """
-    _dummy(message, title)
+    JOptionPane.showMessageDialog(
+        None, message, title, JOptionPane.WARNING_MESSAGE
+    )
