@@ -4,7 +4,10 @@ The following functions give you access to view various Gateway and
 Client data, as well as interact with other various systems.
 """
 
+from __future__ import print_function
+
 __all__ = [
+    "audit",
     "beep",
     "execute",
     "exit",
@@ -25,6 +28,7 @@ __all__ = [
     "getReadTimeout",
     "getSessionInfo",
     "getSystemFlags",
+    "getVersion",
     "invokeAsynchronous",
     "invokeLater",
     "jsonDecode",
@@ -46,22 +50,283 @@ __all__ = [
 ]
 
 import getpass
+import json
 import os
 import platform
-import pprint
+import re
 import sys
 
 import system.__version__ as version
 import system.date
-from com.inductiveautomation.ignition.common import Dataset
-from com.inductiveautomation.ignition.common.script.builtin import (
-    DatasetUtilities,
-)
-from com.inductiveautomation.ignition.common.script.message import Request
-from com.inductiveautomation.ignition.common.util import LoggerEx
+import system.security
 from java.awt import Toolkit
-from java.lang import Thread
+from java.lang import Object, Thread
 from java.util import Date
+from system.dataset import Dataset, PyDataSet
+
+
+class LoggerEx(Object):
+    """This class is a wrapper around a logger which provides additional
+    useful tools.
+
+    To create one, use the newBuilder() function and configure the
+    builder.
+    """
+
+    def getLogger(self):
+        pass
+
+    def getName(self):
+        pass
+
+    @staticmethod
+    def log(*args):
+        pass
+
+
+class Request(Object):
+    def block(self):
+        pass
+
+    def cancel(self):
+        pass
+
+    def checkTimeout(self):
+        pass
+
+    def compose(self, requestWatchers):
+        pass
+
+    def dispatchFunc(self):
+        pass
+
+    def finishExceptionally(self, e):
+        pass
+
+    def finishSuccessfully(self, value):
+        pass
+
+    def get(self):
+        pass
+
+    def getError(self):
+        pass
+
+    def getLongId(self):
+        pass
+
+    def internalOperationCompleted(self):
+        pass
+
+    def internalWait(self):
+        pass
+
+    def onError(self, func):
+        pass
+
+    def onSuccess(self, func):
+        pass
+
+    def startTime(self):
+        pass
+
+
+class Version(Object):
+    """Version object."""
+
+    dev = True
+    snapshot = False
+
+    def __init__(self, major=0, minor=0, rev=0, build=0, beta=0, rc=0):
+        """Version initializer.
+
+        Args:
+            major (int): Major number.
+            minor (int): Minor number.
+            rev (int): Revision number.
+            build (int): Build number.
+            beta (int): Beta number.
+            rc (int): Release Candidate number.
+        """
+        self.major = major
+        self.minor = minor
+        self.rev = rev
+        self.build = build
+        self.beta = beta
+        self.rc = rc
+
+    def __eq__(self, other, strict=False):
+        ret = 0
+        version_1 = [self.major, self.minor, self.rev]
+        version_2 = [other.major, other.minor, other.rev]
+
+        if strict:
+            version_1.append(self.build)
+            version_1.append(self.beta)
+            version_1.append(self.rc)
+            version_2.append(other.build)
+            version_2.append(other.beta)
+            version_2.append(other.rc)
+
+        for i in range(max(len(version_1), len(version_2))):
+            v_1 = version_1[i] if i < len(version_1) else 0
+            v_2 = version_2[i] if i < len(version_2) else 0
+            if v_1 > v_2:
+                ret = 1
+            elif v_1 < v_2:
+                ret = -1
+        return ret
+
+    def __str__(self):
+        return self.toString()
+
+    def compareTo(self, that):
+        """Compares two Versions.
+
+        Note that this comparison is stricter than we want for Gateway
+        restores or project imports. For those, isFutureVersion().
+
+        Args:
+            that (Version): The version to compare.
+
+        Returns:
+             int: 0 if self and that are equal, -1 if that is greater
+                than self, or 1 if self is greater than that.
+        """
+        return self.__eq__(that, True)
+
+    def exists(self):
+        pass
+
+    @staticmethod
+    def fromXML(inputStream):
+        pass
+
+    def getBasicString(self):
+        if self.rc > 0:
+            ret = "{}.{}.{}-rc{}".format(
+                self.major, self.minor, self.rev, self.rc
+            )
+        else:
+            ret = "{}.{}.{}".format(self.major, self.minor, self.rev)
+        return ret
+
+    def getBeta(self):
+        return self.beta
+
+    def getBuildNumber(self):
+        return self.build
+
+    def getMajor(self):
+        return self.major
+
+    def getMinor(self):
+        return self.minor
+
+    def getRc(self):
+        return self.rc
+
+    def getRevision(self):
+        return self.rev
+
+    def getXML(self):
+        pass
+
+    def isDev(self):
+        return self.dev
+
+    def isFutureVersion(self, arg):
+        other = self.parse(arg)
+        return self.__eq__(other) == -1
+
+    def isSnapshot(self):
+        return self.snapshot
+
+    @staticmethod
+    def parse(s):
+        sem_ver = [int(i) for i in re.findall(r"-?\d+", s)]
+        return Version(sem_ver[0], sem_ver[1], sem_ver[2])
+
+    def toParseableString(self):
+        """Returns the version as a compact, parseable (non-XML) string
+        that can be parsed with the 1-string constructor of this class.
+
+        Returns:
+            string: Compact, parseable (non-XML) string.
+        """
+        return "{}.{}.{}.{}".format(
+            self.major, self.minor, self.rev, self.build
+        )
+
+    def toString(self):
+        if self.rc > 0:
+            version = "{}.{}.{}-rc{} (b{})".format(
+                self.major, self.minor, self.rev, self.rc, self.build
+            )
+        elif self.isSnapshot():
+            version = "{}.{}.{}-SNAPSHOT (b{})".format(
+                self.major, self.minor, self.rev, self.build
+            )
+        elif self.build is not None:
+            version = "{}.{}.{} (b{})".format(
+                self.major, self.minor, self.rev, self.build
+            )
+        else:
+            version = "{}.{}.{}".format(self.major, self.minor, self.rev)
+        return version
+
+
+def audit(
+    action=None,
+    actionValue=None,
+    auditProfile="",
+    actor=None,
+    actorHost="localhost",
+    originatingSystem=None,
+    eventTimestamp=None,
+    originatingContext=4,
+    statusCode=0,
+):
+    """Inserts a record into an audit profile.
+
+    Args:
+        action (str): What happened. Default is null. Optional.
+        actionValue (str): What the action happened to. Default is null.
+            Optional.
+        auditProfile (str): Where the audit record should be stored.
+            Defaults to the project's audit profile (if specified), or
+            the gateway audit profile if calling in the gateway or
+            perspective scope. Optional.
+        actor (str): Who made the change. Will be populated
+            automatically if omitted, assuming there is a known user.
+            Optional.
+        actorHost (str): The hostname of whoever made the change. Will
+            be populated automatically if omitted.
+        originatingSystem (object): An even-length list providing
+            additional context to the audit event. Optional.
+        eventTimestamp (Date): When the event happened. Will be set
+            to the current time if omitted. Optional.
+        originatingContext (int): What scope the event originated from:
+            1 means Gateway, 2 means Designer, 4 means Client. Will be
+            set automatically if omitted. Optional.
+        statusCode (int): A quality code to attach to the object.
+            Defaults to 0, indicating no special meaning. Optional.
+    """
+    actor = system.security.getUsername() if actor is None else actor
+    eventTimestamp = (
+        system.date.now() if eventTimestamp is None else eventTimestamp
+    )
+    print(
+        action,
+        actionValue,
+        auditProfile,
+        actor,
+        actorHost,
+        originatingSystem,
+        eventTimestamp,
+        originatingContext,
+        statusCode,
+    )
 
 
 def beep():
@@ -82,11 +347,13 @@ def beep():
 
                 winsound.MessageBeep(winsound.MB_ICONEXCLAMATION)
             except ImportError:
-                pprint.pprint("Beep!")
+                print("Beep!")
         elif platforms[sys.platform] == "macOS":
             os.system('say "beep"')
+        elif platforms[sys.platform] == "Linux":
+            print("\a")
     else:
-        pprint.pprint("Beep!")
+        print("Beep!")
 
 
 def execute(commands):
@@ -101,7 +368,7 @@ def execute(commands):
         commands (list[str]): A list containing the command (1st entry)
             and associated arguments (remaining entries) to execute.
     """
-    pprint.pprint(commands)
+    print(commands)
 
 
 def exit(force=False):
@@ -116,12 +383,12 @@ def exit(force=False):
         force (bool): If True (1), the shutdown-intercept script will be
             skipped. Default is False (0). Optional.
     """
-    pprint.pprint(force)
+    print(force)
 
 
 def getAvailableLocales():
-    """Returns a collection of strings representing the Locales added
-    to the Translation Manager, such as 'en' for English.
+    """Returns a collection of strings representing the Locales added to
+    the Translation Manager, such as 'en' for English.
 
     Returns:
         list[str]: A collection of strings representing the Locales
@@ -200,7 +467,7 @@ def getGatewayAddress():
         str: The address of the Gateway that the client is communicating
             with.
     """
-    return "http://localhost:8088/main"
+    return "http://localhost:8088/"
 
 
 def getGatewayStatus(
@@ -225,7 +492,7 @@ def getGatewayStatus(
         str: A string that indicates the status of the Gateway. A status
             of RUNNING means that the Gateway is fully functional.
     """
-    pprint.pprint([gatewayAddress, connectTimeoutMillis, socketTimeoutMillis])
+    print(gatewayAddress, connectTimeoutMillis, socketTimeoutMillis)
     return "RUNNING"
 
 
@@ -250,8 +517,6 @@ def getInactivitySeconds():
     """Returns the number of seconds since any keyboard or mouse
     activity.
 
-    Note - this function will always return zero in the Designer.
-
     Returns:
         long: The number of seconds the mouse and keyboard have been
             inactive for this client.
@@ -280,7 +545,7 @@ def getLogger(name):
         LoggerEx: A new Logger object used to log informational and
             error messages.
     """
-    pprint.pprint(name)
+    print(name)
     return LoggerEx()
 
 
@@ -366,8 +631,8 @@ def getSessionInfo(usernameFilter=None, projectFilter=None):
         PyDataSet: A dataset representing the Gateway's current
             sessions.
     """
-    pprint.pprint([usernameFilter, projectFilter])
-    return DatasetUtilities.PyDataSet()
+    print(usernameFilter, projectFilter)
+    return PyDataSet()
 
 
 def getSystemFlags():
@@ -387,7 +652,19 @@ def getSystemFlags():
     return 1
 
 
-def invokeAsynchronous(function):
+def getVersion():
+    """Returns the Ignition version number that is currently being run.
+
+    Returns:
+        Version: The currently running Ignition version number. as a
+            Version object.
+    """
+    major, minor, rev = [int(i) for i in version.__version__.split(".")]
+    build = int(version.__build__)
+    return Version(major=major, minor=minor, rev=rev, build=build)
+
+
+def invokeAsynchronous(function, args=None, kwargs=None, description=None):
     """Invokes (calls) the given Python function on a different thread.
 
     This means that calls to invokeAsynchronous will return immediately,
@@ -400,18 +677,30 @@ def invokeAsynchronous(function):
     Args:
         function (object): A Python function object that will get
             invoked with no arguments in a separate thread.
+        args: A list or tuple of Python objects that will be provided to
+            the called function as arguments. Equivalent to the *
+            operator. Optional.
+        kwargs: A dictionary of keyword argument names to Python object
+            values that will be provided to the called function as
+            keyword arguments. Equivalent to the ** operator. Optional.
+        description (str): A description to use for the asynchronous
+            thread. Will be displayed on the current scope's diagnostic
+            view for scripts. For Vision and the Designer, this would be
+            the "Scripts" tab of the Diagnostics popup. For Perspective
+            and the Gateway scope, this would be the Gateway's Running
+            Scripts status page. Optional.
 
     Returns:
         Thread: The executing thread.
     """
-    pprint.pprint(function)
+    print(function, args, kwargs, description)
     return Thread()
 
 
 def invokeLater(function, delay=0):
     """Invokes (calls) the given Python function object after all of the
-    currently processing and pending events are done being processed,
-    or after a specified delay.
+    currently processing and pending events are done being processed, or
+    after a specified delay.
 
     The function will be executed on the GUI, or event dispatch, thread.
     This is useful for events like propertyChange events, where the
@@ -430,7 +719,7 @@ def invokeLater(function, delay=0):
             be invoked after all currently pending events are processed.
             Optional.
     """
-    pprint.pprint([function, delay])
+    print(function, delay)
 
 
 def jsonDecode(jsonString):
@@ -446,8 +735,7 @@ def jsonDecode(jsonString):
     Returns:
         dict: The decoded Python object.
     """
-    pprint.pprint(jsonString)
-    return {"key": "value"}
+    return json.loads(jsonString)
 
 
 def jsonEncode(pyObj, indentFactor=4):
@@ -463,8 +751,7 @@ def jsonEncode(pyObj, indentFactor=4):
     Returns:
         str: The encoded JSON string.
     """
-    pprint.pprint([pyObj, indentFactor])
-    return ""
+    return json.dumps(pyObj, indent=indentFactor)
 
 
 def modifyTranslation(term, translation, locale="en"):
@@ -477,7 +764,7 @@ def modifyTranslation(term, translation, locale="en"):
             identifying the language of the translation. Otherwise, the
             currently set language is used. Optional.
     """
-    pprint.pprint([term, translation, locale])
+    print(term, translation, locale)
 
 
 def playSoundClip(wav, volume=1.0, wait=False):
@@ -496,7 +783,7 @@ def playSoundClip(wav, volume=1.0, wait=False):
             to playSoundClip should wait for the clip to finish before
             it returns. Optional.
     """
-    pprint.pprint([wav, volume, wait])
+    print(wav, volume, wait)
 
 
 def queryAuditLog(
@@ -543,25 +830,21 @@ def queryAuditLog(
     startDate = (
         system.date.addHours(endDate, -8) if startDate is None else startDate
     )
-    pprint.pprint(
-        [
-            auditProfileName,
-            startDate,
-            endDate,
-            actorFilter,
-            actionFilter,
-            targetFilter,
-            valueFilter,
-            systemFilter,
-            contextFilter,
-        ]
+    print(
+        auditProfileName,
+        startDate,
+        endDate,
+        actorFilter,
+        actionFilter,
+        targetFilter,
+        valueFilter,
+        systemFilter,
+        contextFilter,
     )
     return Dataset()
 
 
-def retarget(
-    projectName, gatewayAddress=None, params=None, startupWindows=None
-):
+def retarget(project, addresses=None, params=None, windows=None):
     """This function allows you to programmatically 'retarget' the
     Client to a different project and/or different Gateway.
 
@@ -584,20 +867,23 @@ def retarget(
     _RETARGET_FROM_GATEWAY to the address of the current Gateway.
 
     Args:
-        projectName (str): The name of the project to retarget to.
-        gatewayAddress (str): The address of the Gateway that the
-            project resides on. If omitted, the current Gateway will be
-            used. Format is: "host:httpPort:sslPort/main". Optional.
+        project (str): The name of the project to retarget to.
+        addresses (object): The address of the Gateway that the project
+            resides on. If omitted, the current Gateway will be used.
+            Format is: host:port. Optional.
+            As of 8.0.8 this can be a list of strings. When using a
+            list, the function will try each address in order, waiting
+            for the timeout period between each address attempt.
         params (dict): A dictionary of parameters that will be passed to
             the new project. They will be set as global variables in the
             new project's Python scripting environment. Optional.
-        startupWindows (list[str]): A list of window paths to use as the
+        windows (list[str]): A list of window paths to use as the
             startup windows. If omitted, the project's normal startup
             windows will be opened. If specified, the project's normal
             startup windows will be ignored, and this list will be used
             instead. Optional.
     """
-    pprint.pprint([projectName, gatewayAddress, params, startupWindows])
+    print(project, addresses, params, windows)
 
 
 def sendMessage(
@@ -652,18 +938,16 @@ def sendMessage(
             system that was selected for delivery, where each List item
             is comma-delimited.
     """
-    pprint.pprint(
-        [
-            project,
-            messageHandler,
-            payload,
-            scope,
-            clientSessionId,
-            user,
-            hasRole,
-            hostName,
-            remoteServers,
-        ]
+    print(
+        project,
+        messageHandler,
+        payload,
+        scope,
+        clientSessionId,
+        user,
+        hasRole,
+        hostName,
+        remoteServers,
     )
 
 
@@ -707,15 +991,13 @@ def sendRequest(
     Returns:
         object: The return from the message handler.
     """
-    pprint.pprint(
-        [
-            project,
-            messageHandler,
-            payload,
-            hostName,
-            remoteServer,
-            timeoutSec,
-        ]
+    print(
+        project,
+        messageHandler,
+        payload,
+        hostName,
+        remoteServer,
+        timeoutSec,
     )
 
 
@@ -766,21 +1048,31 @@ def sendRequestAsync(
         Request: The Request object that can be used while waiting for
             the message handler callback.
     """
-    pprint.pprint(
-        [
-            project,
-            messageHandler,
-            payload,
-            hostName,
-            remoteServer,
-            timeoutSec,
-            onSuccess,
-            onError,
-        ]
+    print(
+        project,
+        messageHandler,
+        payload,
+        hostName,
+        remoteServer,
+        timeoutSec,
+        onSuccess,
+        onError,
     )
     return Request()
 
 
+def setConnectTimeout(connectTimeout):
+    """Sets the connect timeout for client-to-gateway communication.
+
+    Specified in milliseconds.
+
+    Args:
+        connectTimeout (int): The new connect timeout, specified in
+            milliseconds.
+    """
+    print(connectTimeout)
+
+
 def setConnectionMode(mode):
     """Sets the connection mode for the client session.
 
@@ -795,19 +1087,7 @@ def setConnectionMode(mode):
         mode (int): The new connection mode. 1 = Disconnected,
             2 = Read-only, 3 = Read/Write.
     """
-    pprint.pprint(mode)
-
-
-def setConnectTimeout(connectTimeout):
-    """Sets the connect timeout for client-to-gateway communication.
-
-    Specified in milliseconds.
-
-    Args:
-        connectTimeout (int): The new connect timeout, specified in
-            milliseconds.
-    """
-    pprint.pprint(connectTimeout)
+    print(mode)
 
 
 def setLocale(locale):
@@ -815,15 +1095,13 @@ def setLocale(locale):
 
     Any valid Java locale code (case-insensitive) can be used as a
     parameter, including ones that have not yet been added to the
-    Translation Manager.
+    Translation Manager. An invalid locale code will cause an Illegal
+    Argument Exception.
 
     Args:
         locale (str): A locale code, such as 'en_US' for US English.
-
-    Raises:
-        IllegalArgumentException: If passed an invalid local code.
     """
-    pprint.pprint(locale)
+    print(locale)
 
 
 def setLoggingLevel(loggerName, loggerLevel):
@@ -838,7 +1116,7 @@ def setLoggingLevel(loggerName, loggerLevel):
         loggerLevel (str): The level you want to change to logger to:
             "trace", "debug", "info", "warn" or "error".
     """
-    pprint.pprint([loggerName, loggerLevel])
+    print(loggerName, loggerLevel)
 
 
 def setReadTimeout(readTimeout):
@@ -850,7 +1128,7 @@ def setReadTimeout(readTimeout):
         readTimeout (int): The new read timeout, specified in
             milliseconds.
     """
-    pprint.pprint(readTimeout)
+    print(readTimeout)
 
 
 def threadDump():
@@ -859,7 +1137,7 @@ def threadDump():
     Returns:
         str: The dump of the current running JVM.
     """
-    return "Ignition version: %s..." % version.__version__
+    return "Ignition version: {}...".format(getVersion())
 
 
 def translate(term, locale=None, strict=False):
@@ -883,5 +1161,5 @@ def translate(term, locale=None, strict=False):
     Returns:
         str: The translated term.
     """
-    pprint.pprint([term, locale, strict])
+    print(term, locale, strict)
     return term
